function agent = EstimatorUpdate(agent,estModel)
% Perform estimator measurement update locally on agent.
% Each agent checks what measurements have been received and incorporates
% them in its state estimate.
% 
% Remark: Compared to the estimator measurement update in the manuscript
% (i.e. equation (34) in Section "Inter-agent differences"), we do not
% explicitly include the disturbance signal d_i here.  The disturbance is
% introduced in the paper for the purpose of analysis; it can model, e.g.
% dropped packets, which are considered in these simulations.
% 
% 
% Inputs:
%   agent       agent's state struct
%   estModel    system model as used for estimation
%
% Output:
%   agent       agent's state struct (updated)
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [10.04.15, ST]    first version
% [02.08.15, ST]    some clean up


%% Estimation: Measurement update

if ~isempty(agent.y_rcv)
    % Some measurement(s) was (were) received...
    
    % received measurements
    ind_y = agent.y_rcv(:,2);   % indices
    y_eff = agent.y_rcv(:,1);       % measurements
    
    % Observer update
    L_eff = agent.L(:,ind_y);
    C_eff = estModel.C(ind_y,:);
    agent.xhat = agent.xpred + L_eff*(y_eff - C_eff*agent.xpred);
    
else
    % No measurement was received; use prediction.
    agent.xhat = agent.xpred;
end;


% Estimate of full state vector
%
% The above estimation can be done with a reduced state model (it is in the
% example of the Balancing Cube).  Irrespective of which model is used in 
% the state estimator, we maintain the full state vector, as it is used 
% for control, for example.

if estModel.flagReducedModel
    % Reduced model is used.  The reduced states are the previous control
    % inputs (cf. technical report, Balancing Cube model); thus, we add those.
    agent.xhat_full = [agent.xhat(1:estModel.n_u); agent.uhat_minus; agent.xhat((estModel.n_u+1):end)];
else
    % Full model is used for state estimation.  Just copy the estimate.
    agent.xhat_full = agent.xhat;
end;