function agent = InitAgents(p_sys,design,flag_randInitEst)
% Configure and initialize agents.
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [09.04.15, ST]    first version
% [20.04.15, ST]    new conventions for measurement/input indices
% [03.05.15, ST]    changed how random initialization is done


%% Prelim
N = p_sys.numAgents;        % Number of agents
n_u = p_sys.n_u;            % signal dimensions
n_x = p_sys.n_x;
n_y = p_sys.n_y;
n_x_e = p_sys.est.n_x;      % signal dimensions (for estimation)


%% Initialize variables

% Agent's state etc.
for i=1:N
    % Sensors/measurements
    nSens = length(p_sys.y_dims{i});    % number of sensors of agent i
    agent(i).nSens = nSens;
    for iS=1:nSens
        agent(i).y{iS} = [];            % local measurement y_i(k)
    end;
    
    % Actuators/inputs
    nInp = length(p_sys.u_dims{i});     % number of actuators of agent i
    agent(i).nInp = nInp;
    for iA=1:nInp
        agent(i).u{iA} = [];            % local control input, u_i(k)
    end;
    
    agent(i).xhat = zeros(n_x_e,1);      % state estimate, \hat{x}_i(k) = \hat{x}_i(k|k)
    agent(i).xpred = zeros(n_x_e,1);     % state prediction, \hat{x}_i(k|k-1)
    agent(i).uhat = zeros(n_u,1);        % local estimate of FULL input vector, \hat{u}^i(k)
    agent(i).uhat_minus = zeros(n_u,1);  % previous input, \hat{u}^i(k-1)
    agent(i).xhat_full = zeros(n_x,1);   % full state vector (used for control)
    agent(i).y_rcv = [];    % received measurements incl. index, [y_i, i]
    agent(i).u_last = [];   % local control input, last communicated to other agents, u_{i,last}(k)
end;

% Randomize initial estimates?
if flag_randInitEst
    for i=1:N
        if ~all(p_sys.est.x_init==0)
            agent(i).xhat = p_sys.est.x_init.*(1*(2*rand(n_x_e,1)-1));
            %agent(i).xhat = p_sys.est.x_init + p_sys.est.x_init.*(0.1*(2*rand(n_x_e,1)-1));
        else
            agent(i).xhat = 1/180*pi*(2*rand(n_x_e,1)-1);
        end;
    end;
else
    for i=1:N
        agent(i).xhat = p_sys.est.x_init;
    end;
end;

% Agent's sensors and inputs, controller/observer gains
for i=1:N
    % Measurements
    for iS=1:agent(i).nSens     % for each sensor
        agent(i).C_i{iS} = p_sys.est.C(p_sys.y_dims{i}{iS},:);  % output matrix corresponding to local sensor
        agent(i).n_y_i{iS} = size(agent(i).C_i{iS},1);          % number of local measurements
        agent(i).delta_y{iS} = design.EB_y.delta{i}{iS};        % delta (one for each sensor)
        agent(i).indices_y{iS} = p_sys.y_dims{i}{iS};           % indices of local measurements
    end;
    
    % Inputs
    for iA=1:agent(i).nInp      % for each actuator
        agent(i).F_i{iA} = design.F(p_sys.u_dims{i}{iA},:);     % controller gain
        agent(i).u{iA} = zeros(size(agent(i).F_i{iA},1),1);     % current u_i(k)
        agent(i).u_last{iA} = zeros(size(agent(i).F_i{iA},1),1);    % last communicated u_i(k)
        agent(i).delta_u{iA} = design.EB_u.delta{i}{iA};
    end;
    
    % Controller and observer gain
    agent(i).F = design.F;      % full controller gain
    agent(i).L = design.L;      % full observer gain
end;


