% (script)
%
% Display the linear model, system properties, controller/estimator design, 
% etc. of the Balancing Cube simulation example.
%
% --
% Supplementary material (simulation examples) to:
%   Distributed Event-based State Estimation (technical report)
%   Event-based State Estimation: An Emulation-based Approach (journal)
%
% Copyright 2015-17 Max Planck Society. All rights reserved.
% 
% Sebastian Trimpe
% Max Planck Institute for Intelligent Systems
% Autonomous Motion Department
% strimpe(at)tuebingen.mpg.de
%
% Revision history
% [04.08.15, ST]    first version

clear;

disp(' ');
disp('=====================================');
disp(' Balancing Cube simulation example');
disp('=====================================');
disp(' ');


%% Get model, controller etc.
% Run configuration and design functions.
p_sys = ConfigureSystemBC();  
design = DesignControllerAndObserverBC(p_sys);


%% Display discrete-time LTI model
disp(' ');
disp(' Linear models');
disp('----------------------------------');

% Model
disp(' ');
disp('Discrete-time state-space model (used for controller design).');
disp('  x(k) = A x(k-1) + B u(k-1) + v(k-1) ');
disp('  y(k) = C x(k) + w(k) ');
disp('with...');
disp(' ');
disp('A = ');
disp(p_sys.A);

disp('B = ');
disp(p_sys.B);

disp('C = ');
disp(p_sys.C);

disp('Sampling time (s) = ');
disp(p_sys.Ts);

disp(' ');


% Reduced model used for estimator
% Model
disp(' ');
disp('Reduced state-space model (used for estimator design).');
disp('  x(k) = A x(k-1) + B1 u(k-1) + B2 u(k-2) + v(k-1) ');
disp('  y(k) = C x(k) + w(k) ');
disp('with...');
disp(' ');
disp('A = ');
disp(p_sys.est.A);

disp('B1 = ');
disp(p_sys.est.B);

disp('B2 = ');
disp(p_sys.est.Bminus);

disp('C = ');
disp(p_sys.est.C);

disp(' ');
disp(' ');


%% System properties
disp(' ');
disp(' System properties');
disp('----------------------------------');
disp(' ');

% Controllability & observability

% Controllability from inputs of one agent
for iA = 1:p_sys.numAgents
    disp(['Controllability from Agent ',num2str(iA),':']);
    rkCtrb = rank(ctrb(p_sys.A,p_sys.B(:,p_sys.u_dims{iA}{1})));
    disp([' Rank: ',num2str(rkCtrb),' / ',num2str(length(p_sys.A))]);
end;
disp(' ');

% Observability from sensors of one agent
for iA = 1:p_sys.numAgents
    disp(['Observability from Agent ',num2str(iA),':']);
    rkObsv = rank(obsv(p_sys.est.A,p_sys.est.C([p_sys.y_dims{iA}{:}],:)));
    disp([' Rank: ',num2str(rkObsv),' / ',num2str(length(p_sys.est.A))]);
end;


% Observability & controllability from all sensors and inputs
disp(' ');
disp(['Controllability from all inputs:']);
rkCtrb = rank(ctrb(p_sys.A,p_sys.B));
disp([' Rank: ',num2str(rkCtrb),' / ',num2str(length(p_sys.A))]);
disp(['Observability from all outputs:']);
rkObsv = rank(obsv(p_sys.est.A,p_sys.est.C));
disp([' Rank: ',num2str(rkObsv),' / ',num2str(length(p_sys.est.A))]);

disp(' ');
disp(' ');


%% Display noise and disturbance properties
disp(' ');
disp(' Noise & disturbance properties');
disp('----------------------------------');

% Process disturbance
% plot process disturbance
fig_dist = figure;
d_scaled = p_sys.dist.E*p_sys.dist.d_signal';
tt = (0:size(d_scaled,2)-1)*p_sys.Ts;
for i=1:size(d_scaled,1)
    subplot(size(d_scaled,1),1,i);
    plot(tt,d_scaled(i,:)');
    grid;
    ylabel(['Disturbance v_',num2str(i)]);
end;
xlabel('Time t (s)');
%
disp(' ');
disp(['Process disturbance v(k):  impulsive input disturbance, see Figure.']);


% Process noise
disp(' ');
disp('Process noise (on inputs) v(k) = B*vu(k) with vu(k):  uniformly distributed in [-vu_lim, +vu_lim]');
disp('vu_lim=');
disp(p_sys.vInput_lim);

% Measurement noise
disp(' ');
disp('Sensor noise w(k):  uniformly distributed in [-w_lim, +w_lim]');
disp('w_lim=');
disp(p_sys.w_lim);

disp(' ');
disp(' ');


%% Observer
disp(' ');
disp(' Observer design');
disp('----------------------------------');

% Observer gain and eigenvalues
disp(' ');
disp('Linear observer gain:');
disp('L=');
disp(design.L);
disp(' ');
disp('Eigenvalues of centralized observer dynamics, (I-LC)A:');
disp(eig(design.closedLoopDynEst));

disp(' ');
disp(' ');


%% Controller
disp(' ');
disp(' Controller design');
disp('----------------------------------');

% State-feedback gain and eigenvalues
disp(' ');
disp('State-feedback gain:');
disp('F=');
disp(design.F);
disp(' ');
disp('Eigenvalues of centralized state-feedback design, A+BF:');
disp(eig(design.closedLoopDynCtrl));

disp(' ');
disp(' ');


%% Triggering thresholds
disp(' ');
disp(' Communication design');
disp('----------------------------------');

disp(' ');
disp('Triggering thresholds');
disp(['  ... for the measurements y1, y2, ..., y9:']);
disp([[design.EB_y.delta{1}{:}]'; ...
    [design.EB_y.delta{2}{:}]'; ...
    [design.EB_y.delta{3}{:}]'; ...
    [design.EB_y.delta{4}{:}]'; ...
    [design.EB_y.delta{5}{:}]'; ...
    [design.EB_y.delta{6}{:}]']);
disp(' ');
disp(['  ... for the inputs u1, u2, ..., u6:']);
disp([[design.EB_u.delta{1}{:}]'; ...
    [design.EB_u.delta{2}{:}]'; ...
    [design.EB_u.delta{3}{:}]'; ...
    [design.EB_u.delta{4}{:}]'; ...
    [design.EB_u.delta{5}{:}]'; ...
    [design.EB_u.delta{6}{:}]']);
disp(' ');
disp('Synchronous averaging period:');
disp(['K = ',num2str(design.syncAvg.K)]);

disp(' ');
disp(' ');
    